from __future__ import absolute_import
from __future__ import unicode_literals

from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.util import status_me


class NlpController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(NlpController, self).__init__(check_result, service_path)

    @status_me("nlp")
    def push_nlp_images(self):
        self.push_images(self.project)

    @status_me("nlp")
    def deploy_nlp_configmap(self):
        self.deploy_all_configmap()

    @status_me("nlp")
    def start_nlp_service(self):
        self._create_namespace(namespaces=self.namespaces,
                               istio_injection_namespaces=self.istio_injection_namespaces)
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("nlp")
    def prepare_nlp_data(self):
        self.prepare_data(project=self.project)

    def run(self):
        self.push_nlp_images()
        self.deploy_nlp_configmap()
        self.start_nlp_service()
        self.prepare_nlp_data()
        self.project_pod_check()




from __future__ import absolute_import
from __future__ import unicode_literals

import os
import json
from minio import Minio
from laipvt.helper.exception import UtilsError
from laipvt.interface.serviceinterface import ServiceInterface
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me
from laipvt.sysutil.command import CREATE_DB
from laipvt.sysutil.conf import AccountIdConfig
from laipvt.model.sql import SqlModule
from laipvt.sysutil.command import HIDE_MENU_SQL


class MageController(ServiceInterface):
    def __init__(self, check_result, service_path):
        super(MageController, self).__init__(check_result, service_path)

        self.nginx_template = path_join(self.templates_dir, "nginx/http/nginx-mage.tmpl")
        self.nginx_tmp = path_join("/tmp", "nginx-mage.conf")
        self.nginx_file_remote = path_join(self.deploy_dir, "nginx/http/nginx-mage.conf")
        self.minio_data_list = [
            path_join(self.data_dir, "mage_minio_data"),
            path_join(self.data_dir, "siber_minio_data")
        ]

    @status_me("mage")
    def deploy_configmap(self):
        self.deploy_all_configmap()

    @status_me("mage")
    def deploy_mage_istio(self):
        self.deploy_istio()

    @status_me("mage")
    def init_minio_data(self):
        self.init_minio()
        # try:
        #     for bucket in self.service_info.buckets:
        #         # print(bucket)
        #         try:
        #             endpoint = "{}:{}".format(self.middleware_cfg.minio.lb, self.middleware_cfg.minio.nginx_proxy_port)
        #             cli = Minio(
        #                 endpoint,
        #                 self.middleware_cfg.minio.username,
        #                 self.middleware_cfg.minio.password,
        #                 secure=False
        #             )
        #             if not cli.bucket_exists(bucket):
        #                 cli.make_bucket(bucket)
        #             content_types = {
        #                 'txt': 'text/plain',
        #                 'jpg': 'image/jpg',
        #                 'gif': 'image/gif',
        #                 'png': 'image/png',
        #                 'jpeg': 'image/jpeg',
        #                 'pdf': 'application/pdf',
        #                 'tif': 'image/tiff',
        #                 'tiff': 'image/tiff',
        #                 'bmp': 'image/bmp',
        #                 'xlsx': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
        #             }
        #             for data_dir in self.minio_data_list:
        #                 for i in os.listdir(data_dir):
        #                     image_name = path_join(data_dir, i)
        #                     file_type = content_types[i.split('.')[-1]]
        #                     # cli.fput_object(bucket, i, image_name, content_type=file_type)
        #                     # if i.split('.')[-1] == "xlsx":
        #                     cli.fput_object(bucket, "document-mining-backend/" + i,  image_name, content_type=file_type)
        #
        #             policy_read_write = {
        #                 "Version": "2012-10-17",
        #                 "Statement": [
        #                     {
        #                         "Action": [
        #                             "s3:GetObject"
        #                         ],
        #                         "Effect": "Allow",
        #                         "Principal": "*",
        #                         "Resource": [
        #                             "arn:aws:s3:::{}/*".format(bucket)
        #                         ],
        #                         "Sid": ""
        #                     }
        #                 ]
        #             }
        #             cli.set_bucket_policy(bucket, json.dumps(policy_read_write))
        #
        #         except Exception as e:
        #             log.error(e)
        #             log.error("Minio上传数据失败")
        #             exit(2)
        # except Exception as e:
        #     log.error(e)
        #     log.error("创建bucket失败")
        #     exit(2)

    def create_mage_db_if_exist(self):
        if self.middleware_cfg.mysql.is_deploy:
            mysql_host = self.master_host.ipaddress
        else:
            mysql_host = self.middleware_cfg.mysql.ipaddress[0]
        sql = SqlModule(host=mysql_host, port=int(self.middleware_cfg.mysql.port),
                        user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        if not sql.select("show databases like '%_saas_docuds';"):
            create_db = CREATE_DB.format(db_name="laiye_saas_docuds")
            sql.insert_sql(create_db)

    @status_me("mage")
    def init_mage_mysql(self):
        log.info("初始化mage mysql")
        version = self.private_deploy_version.split("-")[0]

        self.create_mage_db_if_exist()
        cmd = "docker run --rm --entrypoint /home/works/program/mageTool \
        {}/{}/document-mining-innerservice:{} --operateMode dbnew \
        --host {} --user {} --password {} --port {} --toVersion {}".format(
            self.registry_hub, self.project, self.private_deploy_version,
            self.middleware_cfg.mysql.ipaddress[0], self.middleware_cfg.mysql.username, self.middleware_cfg.mysql.password, self.middleware_cfg.mysql.port,
            version
        )
        res = self._exec_command_to_host(cmd=cmd, server=self.servers[0], check_res=True)
        if res["code"] != 0:
            log.error("初始化mage mysql失败")
            log.error(res["stdout"])
            exit(2)
        log.info("初始化mage mysql完成")
        #self.init_mysql(sql_path=self.service_path.sqls)
        # 渲染siber sqls目录
        # FileTemplate(self.middleware_cfg, self.service_path.siber_sqls, self.service_path.siber_sqls_ok).fill()
        # self.init_mysql(sql_path=self.service_path.siber_sqls_ok)

    @status_me("mage")
    def mage_transfer_data(self):
        src_dir_path = path_join(self.service_path.data, "mage_transfer_data")
        dest_dir_path = path_join(self.deploy_dir, "mage_transfer_data")
        image = "{}/{}/document-mining-innerservice:{}".format(self.registry_hub, self.project, self.private_deploy_version)
        version = self.private_deploy_version.split("-")[0]

        cmd = "docker run --rm --entrypoint /home/works/program/mageTool {image} \
         --operateMode datatransfer --dataMode 2 --minioAddr http://{minio_host}:{minio_port} \
          --host {mysql_host} --user {mysql_user} --password {mysql_password} --port {mysql_port} \
          --toVersion {version}".format(
            image=image, minio_host=self.middleware_cfg.minio.lb, minio_port=self.middleware_cfg.minio.port,
            mysql_host=self.middleware_cfg.mysql.ipaddress[0],mysql_user=self.middleware_cfg.mysql.username,
            mysql_password=self.middleware_cfg.mysql.password, mysql_port=self.middleware_cfg.mysql.port,
            version=version
        )
        self._send_file(src=src_dir_path, dest=dest_dir_path)
        res = self._exec_command_to_host(cmd=cmd, server=self.servers[0], check_res=True)
        log.info("userid: {}".format(res["stdout"].split("\n")[-1]))
        AccountIdConfig().save(res["stdout"].split("\n")[-1])

    @status_me("mage")
    def hide_menu(self):
        if self.middleware_cfg.mysql.is_deploy:
            mysql_host = self.master_host.ipaddress
        else:
            mysql_host = self.middleware_cfg.mysql.ipaddress[0]
        hide_menu_id = ",".join(self.admin_config.hide_menu_id)

        # sql = SqlModule(host=mysql_host, port=int(self.middleware_cfg.mysql.port),
        #                 user=self.middleware_cfg.mysql.username, passwd=self.middleware_cfg.mysql.password)
        # sql.insert_sql(HIDE_MENU_SQL.format(hide_menu_id=hide_menu_id))
        image = "{}/{}/document-mining-innerservice:{}".format(self.registry_hub, self.project,
                                                               self.private_deploy_version)
        version = self.private_deploy_version.split("-")[0]
        cmd = "docker run --rm --entrypoint /home/works/program/mageTool {image} \
             --operateMode recordauthorize --toVersion {version}  --fromVersion v2.1 \
              --host {mysql_host} --user {mysql_user} --password {mysql_password} --port {mysql_port} --name " \
              "laiye_saas_docuds --engineStrs AI_ENGINE_OCR_GENERAL_TEXT_GPU --menuUids {hide_menu_id}".format(
            image=image, minio_host=self.middleware_cfg.minio.lb, minio_port=self.middleware_cfg.minio.port,
            mysql_host=mysql_host, mysql_user=self.middleware_cfg.mysql.username,
            mysql_password=self.middleware_cfg.mysql.password, mysql_port=self.middleware_cfg.mysql.port,
            version=version, hide_menu_id=hide_menu_id
        )
        res = self._exec_command_to_host(cmd=cmd, server=self.servers[0], check_res=True)
        log.info("userid: {}".format(res["stdout"].split("\n")[-1]))
        log.info(res["code"])

    @status_me("mage")
    def push_mage_images(self):
        self.push_images(self.project)

    @status_me("mage")
    def start_mage_service(self):
        self.start_service(project=self.project, version=self.private_deploy_version)

    @status_me("mage")
    def mage_proxy_on_nginx(self):
        self.proxy_on_nginx(self.nginx_template, self.nginx_tmp, self.nginx_file_remote)

    @status_me("mage")
    def init_mage_entuc_clients(self):
        self.init_usercenter_clients(project=self.project)

    @status_me("mage")
    def apply_mage_model_svc(self):
        model_svc_src = path_join(self.templates_dir, "model-svc")
        model_svc_dest = path_join(self.deploy_dir, "model-svc")
        self._send_file(src=model_svc_src, dest=model_svc_dest)
        cmd = "kubectl apply -R -f {}".format(model_svc_dest)
        self._exec_command_to_host(cmd=cmd, server=self.harbor_hosts[0])

    def run(self):
        self.push_mage_images()
        self.init_mage_mysql()
        # self.init_identity_user()
        self.init_minio_data()
        self.deploy_configmap()
        self.deploy_mage_istio()
        # self.mage_transfer_data()
        self.start_mage_service()
        self.mage_proxy_on_nginx()
        self.project_pod_check()
        self.hide_menu()
        self.apply_mage_model_svc()
        self.init_mage_entuc_clients()




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import os
import re
import json
from laipvt.helper.errors import Helper
from laipvt.sysutil.gvalue import CHECK_FILE, PROJECT_INFO_FILE
from laipvt.sysutil.util import find
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.packagehandler import DeployPackageHandler
from laipvt.controller.kubernetes.kube import KubeController
from laipvt.controller.middleware.harbor import HarborController
from laipvt.controller.middleware.nginx import NginxController
from laipvt.controller.middleware.etcd import EtcdController
from laipvt.controller.middleware.minio import MinioController
from laipvt.controller.middleware.redis import RedisController
from laipvt.controller.middleware.mysql import MysqlController
from laipvt.controller.middleware.elasticsearch import EsController
from laipvt.controller.middleware.rabbitmq import RabbitmqController
from laipvt.controller.middleware.siber import SiberController
from laipvt.controller.service.license import LicenseController
from laipvt.controller.service.mage import MageController
from laipvt.controller.service.ocr_standard import OcrStandardController
from laipvt.controller.service.nlp import NlpController
from laipvt.controller.service.seal import SealController
from laipvt.controller.service.captcha import CaptchaController
from laipvt.controller.service.commander import CommanderController
from laipvt.controller.service.ocr import OcrController
from laipvt.controller.service.chatbot import ChatbotController
from laipvt.controller.service.usercenter import UserCenterController
from laipvt.controller.service.dataservice import DataServiceController
from laipvt.controller.service.notice import NoticeController
from laipvt.controller.service.ocr_text import OcrTextController
from laipvt.controller.service.ocr_text_cpu import OcrTextCpuController
from laipvt.controller.service.ocr_table import OcrTableController
from laipvt.controller.service.ocr_idcard_server import OcrIdcardController
from laipvt.controller.service.rpa_collaboration import RpaCollaborationController
from laipvt.controller.service.ocr_layout_analysis import OcrLayoutAnalysisController
from laipvt.controller.service.ocr_bizlicense import OcrBizlicenseController
from laipvt.controller.service.nlp_layoutlm import NlpLaYouTlmController
from laipvt.controller.middleware.monitor import MonitorController
from laipvt.controller.middleware.keepalived import KeepalivedController
from laipvt.handler.middlewarehandler import EtcdConfigHandler, MysqlConfigHandler, EsConfigHandler, \
    MinioConfigHandler, RabbitmqConfigHandler, RedisConfigHandler, HarborConfigHandler, NginxConfigHandler, \
    IdentityConfigHandler, SiberConfigHandler, OcrConfigHandler, MonitorConfigHandler, KeepalivedConfigHandler
from laipvt.sysutil.util import write_to_file, read_form_json_file, gen_https_self_signed_ca, modify_umask
from laipvt.sysutil.kube_common import wait_pod_running
from laipvt.sysutil.print_info import print_project_info
from laipvt.sysutil.check import is_pre_check, check_use_https_ca, check_https_ca_self_signed

def deploy_main(args):
    # 获取前置检查结果
    global service_path
    check_result_file = CHECK_FILE
    check_result = CheckResultHandler(check_result_file)

    if args.targzFile:
        if not args.isCheck:
            if not is_pre_check():
                print(Helper().PRECHECK_FAILED)
                exit(2)

        # 设置umask
        modify_umask(check_result=check_result, is_set=True)
        if not check_use_https_ca():
            print(Helper().HTTPS_CERTS_ERROR)
            exit(2)

        if check_https_ca_self_signed():
            gen_https_self_signed_ca()

        pkg_path = False
        if not os.path.exists(args.targzFile):
            cwd = [os.getcwd(), check_result.deploy_dir]
            for d in cwd:
                pkg_path = find(d, args.targzFile, file=True)
                if pkg_path:
                    break
        else:
            pkg_path = os.path.join(os.getcwd(), args.targzFile)
        if not pkg_path:
            print(Helper().FILE_NOT_FOUND)
            exit(2)
        PKG = os.path.dirname(pkg_path)
        ID = os.path.basename(pkg_path).split(".")[0]
        PKG_DIR = pkg_path.split(".")[0]

        # 将项目ID和path写入文件缓存
        project_dict = {"PKG": PKG, "ID": ID}
        write_to_file(PROJECT_INFO_FILE, json.dumps(project_dict, indent=4))
        deploy_package = DeployPackageHandler(PKG, ID)
        if not os.path.exists(PKG_DIR):
            deploy_package.unpack()



        # 解析部署包
        parse_package = deploy_package.parse()
        kubernetes_package = parse_package.kubernetes
        middleware_package = parse_package.middleware
        harbor_package = parse_package.harbor

        if not os.path.exists(os.path.join(PKG_DIR, "kubernetes")):
            kubernetes_package.kubernetes_unpack()
        if not os.path.exists(os.path.join(PKG_DIR, "middleware")):
            middleware_package.unpack()
        if not os.path.exists(os.path.join(PKG_DIR, "harbor")):
            harbor_package.unpack()

        # install harbor
        haror_path = harbor_package.parse().harbor
        harbor_config = HarborConfigHandler()
        harbor = HarborController(check_result, harbor_config, haror_path)
        harbor.install_harbor()

        # install nginx
        nginx_package = middleware_package.parse().nginx
        nginx_config = NginxConfigHandler()
        nginx = NginxController(check_result, nginx_config, nginx_package)
        nginx.install_nginx()

        # install kubernetes
        if not args.sikpKubernetes:
            kube_info = kubernetes_package.parse()
            kube = KubeController(check_result, kube_info)
            kube.add_hosts()
            kube.system_prepare()
            kube.init_primary_master()
            kube.cp_kube_config()
            kube.kube_completion()
            kube.install_network_plugin()
            kube.join_master()
            kube.join_node()
            kube.install_helm()
            kube.install_istio()
            kube.install_rook()
            kube.install_nvidia_device_plugin()
            kube.install_chaosblade()
            # 更新nginx服务tcp代理apiserever cluster
            nginx.renew_apiserver_config()
            # 临时解决istio vs 命名空间问题
            kube.create_namespace()


        # install service
        services = {
            "license": LicenseController,
            "mage": MageController,
            "entcmd": CommanderController,
            "chatbot": ChatbotController,
            "nlp": NlpController,
            "captcha": CaptchaController,
            "ocr": OcrStandardController,
            "ocr_3rd": OcrController,
            "ocr_seal": SealController,
            "entuc": UserCenterController,
            "dataservice": DataServiceController,
            "notice": NoticeController,
            "ocr_text": OcrTextController,
            "ocr_text_cpu": OcrTextCpuController,
            "ocr_table": OcrTableController,
            "ocr_idcard": OcrIdcardController,
            "rpa-collaboration": RpaCollaborationController,
            "ocr_layout_analysis": OcrLayoutAnalysisController,
            "ocr_bizlicense": OcrBizlicenseController,
            "nlp_layoutlm": NlpLaYouTlmController,
        }

        middlewares = {
            "etcd": (EtcdConfigHandler, EtcdController),
            "minio": (MinioConfigHandler, MinioController),
            "redis": (RedisConfigHandler, RedisController),
            "mysql": (MysqlConfigHandler, MysqlController),
            "elasticsearch": (EsConfigHandler, EsController),
            "rabbitmq": (RabbitmqConfigHandler, RabbitmqController),
        }

        for s in parse_package.service:
            dir_dict = {}
            for i in os.listdir(PKG_DIR):
                # 匹配项目名称，找出部署包
                # chatbot_1.9-20221026191309-test.tar.gz
                project_name_find = re.findall(r'(.*)_\d\.{0,3}\d{0,3}-\d{14}-.*.tar.gz', i)
                if project_name_find:
                    dir_dict[project_name_find[0]] = i

            if s.project_name in dir_dict:
                s.unpack(package_name=dir_dict[s.project_name])
                service_path = s.parse()

                # 遍历服务需要的中间件列表，依次安装
                middleware_list = service_path.config.middleware
                middleware_list.insert(0, "etcd")

                for mid in middleware_list:
                    path = middleware_package.parse()[mid]
                    config = middlewares[mid][0]()
                    middleware = middlewares[mid][1](check_result, config, path)
                    middleware.deploy()

                if s.project_name == "ocr_3rd":
                    ocr_handler = OcrConfigHandler()
                    deploy_service = services[s.project_name](service_path, check_result, ocr_handler, s.root_dir)
                elif s.project_name == "license":
                    deploy_service = services[s.project_name](check_result, service_path, PKG_DIR)
                    deploy_service.run()
                else:
                    deploy_service = services[s.project_name](check_result, service_path)
                deploy_service.run()
        # deploy_service = services["license"](check_result, "", PKG_DIR)
        # deploy_service.runqrcode(PKG_DIR)
        # check all pod status
        if not wait_pod_running:
            print("kubernetes集群中有pod启动状态异常，请检查: kubectl get pod -A")
            exit(2)

        # install siber
        # siber_path = middleware_package.parse().siber
        # siber_config = SiberConfigHandler()
        # siber = SiberController(check_result, siber_config, siber_path)
        # siber.deploy_siber()
        # for s in parse_package.service:
        #     if s.project_name == "mage":
        #         siber.replace_mage_collection_tag(parse_package.config.siber_tags)
        #     elif s.project_name == "commander":
        #         exit()

        # 设置umask
        modify_umask(check_result=check_result, is_set=False)
        # 打印帮助信息
        print_project_info()
        # 打印特征码
        deploy_service = services["license"](check_result, service_path, PKG_DIR)
        deploy_service.runqrcode(PKG_DIR)

    if args.which == "license":
        if args.LicenseFile:
            if os.path.exists(args.LicenseFile):
                # print(args.LicenseFile)
                project_dict = read_form_json_file(PROJECT_INFO_FILE)

                # 解析大包
                deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
                parse_package = deploy_package.parse()
                license_package = parse_package.license
                # license_package.unpack()
                license_path = license_package.parse()
                license = LicenseController(check_result, license_path)
                license.renew_license(license_file=args.LicenseFile)
            else:
                print("请检查指定的新授权文件是否存在: {}".format(args.LicenseFile))
                exit(1)

        if args.OcrLicenseFile:
            if os.path.exists(args.OcrLicenseFile):
                project_dict = read_form_json_file(PROJECT_INFO_FILE)

                deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
                parse_package = deploy_package.parse()
                renew_flag = False
                for s in parse_package.service:
                    if s.project_name == "ocr":
                        service_path = s.parse()
                        ocr_handler = OcrConfigHandler()
                        deploy_ocr = OcrController(service_path, check_result, ocr_handler, s.root_dir)
                        deploy_ocr.renew_license(license_file=args.OcrLicenseFile)
                        renew_flag = True

                if not renew_flag:
                    print("OCR license未更新成功，请检查是否存在合合服务")
                    exit(2)

    if args.which == "add":
        if args.Monitor:
            project_dict = read_form_json_file(PROJECT_INFO_FILE)
            deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
            parse_package = deploy_package.parse()
            middleware_package = parse_package.middleware

            monitor_path = middleware_package.parse().monitor
            monitor_config = MonitorConfigHandler()
            monitor = MonitorController(check_result, monitor_config, monitor_path)
            monitor.deploy_monitor()

        if args.Keepalive:
            project_dict = read_form_json_file(PROJECT_INFO_FILE)
            deploy_package = DeployPackageHandler(project_dict["PKG"], project_dict["ID"])
            parse_package = deploy_package.parse()
            middleware_package = parse_package.middleware

            keepalive_path = middleware_package.parse().keepalived
            keepalive_config = KeepalivedConfigHandler()
            keepalive = KeepalivedController(check_result, keepalive_config, keepalive_path)
            keepalive.deploy_keepalived()

    if args.which == "resetdata":
        if args.Service:
            print(args.Service)

            package_info = read_form_json_file(PROJECT_INFO_FILE)
            deploy_package = DeployPackageHandler(package_info["PKG"], package_info["ID"])
            PKG_DIR = os.path.join(package_info["PKG"], package_info["ID"])

            # 解析
            parse_package = deploy_package.parse()
            service_package = parse_package.service

            for s in service_package:
                dir_dict = {}
                for i in os.listdir(PKG_DIR):
                    # 匹配项目名称，找出服务部署包
                    project_name_find = re.findall(r'(.*)-\d\.{0,3}\d{0,3}_\d{14}.tar.gz', i)
                    if project_name_find:
                        dir_dict[project_name_find[0]] = i

                if s.project_name in dir_dict:
                    s.unpack(package_name=dir_dict[s.project_name])
                    service_path = s.parse()
                    middleware_list = service_path.config.middleware

                    middlewares = {
                        "etcd": (EtcdConfigHandler, EtcdController),
                        "minio": (MinioConfigHandler, MinioController),
                        "redis": (RedisConfigHandler, RedisController),
                        "mysql": (MysqlConfigHandler, MysqlController),
                        "elasticsearch": (EsConfigHandler, EsController),
                        "rabbitmq": (RabbitmqConfigHandler, RabbitmqController),
                    }

                    for mid in middleware_list:
                        path = middleware_package.parse()[mid]
                        config = middlewares[mid][0]()
                        middleware = middlewares[mid][1](check_result, config, path)
                        middleware.remove()
                        middleware.deploy()

                    if args.Service == "chatbot":
                        deploy_service = ChatbotController(check_result, service_path)
                        deploy_service.init_chatbot_mysql.set_force("chatbot", True)
                        deploy_service.init_chatbot_mysql()
                        deploy_service.project_pod_check()
                        deploy_service.init_chatbot_entuc_clients.set_force("chatbot", True)
                        deploy_service.init_chatbot_entuc_clients()
                        exit(0)
                    elif args.Service == "entcmd":
                        deploy_service = CommanderController(check_result, service_path)
                        deploy_service.init_commander_mysql.set_force("entcmd", True)
                        deploy_service.init_commander_mysql()
                        deploy_service.init_commander_rabbitmq.set_force("entcmd", True)
                        deploy_service.init_commander_rabbitmq()
                        deploy_service.init_commander_minio.set_force("entcmd", True)
                        deploy_service.init_commander_minio()
                        deploy_service.project_pod_check()
                        exit(0)
                    elif args.Service == "rpa-collaboration":
                        deploy_service = RpaCollaborationController(check_result, service_path)
                        exit(0)
                    elif args.Service == "notice":
                        deploy_service = NoticeController(check_result, service_path)
                        exit(0)
    if args.which == "test":
        if args.Service:
            print(args.Service)

            package_info = read_form_json_file(PROJECT_INFO_FILE)
            deploy_package = DeployPackageHandler(package_info["PKG"], package_info["ID"])
            PKG_DIR = os.path.join(package_info["PKG"], package_info["ID"])

            # 解析
            parse_package = deploy_package.parse()
            service_package = parse_package.service

            for s in service_package:
                dir_dict = {}
                for i in os.listdir(PKG_DIR):
                    # 匹配项目名称，找出服务部署包
                    project_name_find = re.findall(r'(.*)-\d\.{0,3}\d{0,3}_\d{14}.tar.gz', i)
                    if project_name_find:
                        dir_dict[project_name_find[0]] = i

                if s.project_name in dir_dict:
                    s.unpack(package_name=dir_dict[s.project_name])
                    service_path = s.parse()
                    # print(service_path)

                    if args.Service == "chatbot":
                        deploy_service = ChatbotController(check_result, service_path)
                        deploy_service.run_apptest()
                        exit(0)
                    elif args.Service == "entcmd":
                        deploy_service = CommanderController(check_result, service_path)
                        deploy_service.run_apptest()
                        exit(0)
                    elif args.Service == "rpa-collaboration":
                        deploy_service = RpaCollaborationController(check_result, service_path)
                        deploy_service.run_apptest()
                        exit(0)
                    elif args.Service == "notice":
                        deploy_service = NoticeController(check_result, service_path)
                        deploy_service.run_apptest()
                        exit(0)



from __future__ import absolute_import
from __future__ import unicode_literals

import os
import requests
from minio import Minio
from laipvt.interface.middlewareinterface import MiddlewareInterface
from laipvt.handler.confighandler import CheckResultHandler
from laipvt.handler.middlewarehandler import MinioConfigHandler
from laipvt.sysutil.template import FileTemplate
from laipvt.sysutil.util import path_join, log, status_me, ssh_obj
from laipvt.helper.errors import Helper
from laipvt.model.server import ServerModel


class MinioController(MiddlewareInterface):
    def __init__(self, result: CheckResultHandler, handler: MinioConfigHandler, template: str):
        super(MinioController, self).__init__(result, handler, template)
        self.minio_cfg = MinioConfigHandler().get_config_with_check_result()
        self.minio_nginx_tmp = path_join("/tmp", "nginx-minio.conf")
        self.minio_nginx_template = path_join(self.template, "nginx-minio.tmpl")
        self.minio_nginx_file = path_join(self.deploy_dir, "nginx/http/nginx-minio.conf")
        self.minio_cfg["minio"]["ipaddress"] = self.handler.cfg["ipaddress"]

        self.docker_conf_tmpl = path_join(self.template, "init_readonly_user.sh")
        self.docker_conf_file_tmp = path_join("/tmp", "init_readonly_user.sh")
        self.docker_conf_file = path_join(self.base_dir, "init_readonly_user.sh")
        
        self.minio_service_file_template = path_join(self.template, "minio_service.tmpl")
        self.minio_service_file_dest = path_join("/tmp", "minio_service.yaml")
        self.minio_service_file_remote = path_join(self.base_dir, "svc", "minio_service.yaml")


    def generate_conf_file(self) -> bool:
        FileTemplate(self.minio_cfg, self.docker_conf_tmpl, self.docker_conf_file_tmp).fill()
        return True if os.path.isfile(self.docker_conf_file_tmp) else False

    def send_docker_conf_file(self):
        for server in self.master_server:
            log.info(Helper().SEND_FILE.format(self.docker_conf_file_tmp, server.ipaddress, self.docker_conf_file))
            ssh_cli = ssh_obj(ip=server.ipaddress, user=server.username, password=server.password, port=server.port)
            try:
                ssh_cli.put(self.docker_conf_file_tmp, self.docker_conf_file)
            except Exception as e:
                log.error(e)
                exit(2)
            finally:
                ssh_cli.close()

    def _proxy_on_nginx(self):
        FileTemplate(self.minio_cfg, self.minio_nginx_template, self.minio_nginx_tmp).fill()
        self.update_nginx_config()
        self.generate_docker_compose_file(self.minio_cfg)

    def _create_bucket(self, bucket: str):
        log.info(Helper().CREATE_BUCKET.format(bucket))
        try:
            self.endpoint = "{}:{}".format(self.minio_cfg["minio"]["lb"], self.minio_cfg["minio"]["nginx_proxy_port"])
            cli = Minio(
                self.endpoint,
                self.minio_cfg["minio"]["username"],
                self.minio_cfg["minio"]["password"],
                secure=False
            )
            if not cli.bucket_exists(bucket):
                cli.make_bucket(bucket)
                return True
        except Exception as e:
            log.error(e)
            exit(2)

    def create_minio_service_kubernetes(self):
        server = ServerModel(self.harbor_server)
        FileTemplate(self.minio_cfg, self.minio_service_file_template, self.minio_service_file_dest).fill()
        server.send_file(self.minio_service_file_dest, self.minio_service_file_remote)

        cmd = "kubectl apply -f {}".format(self.minio_service_file_remote)
        ssh_cli = ssh_obj(ip=self.harbor_server[0].ipaddress, user=self.harbor_server[0].username,
                          password=self.harbor_server[0].password, port=self.harbor_server[0].port)
        results = ssh_cli.run_cmd(cmd)
        if results["code"] != 0:
            log.error("{} {}".format(results["stdout"], results["stderr"]))
            exit(2)
            
    def check(self):
        super().wait_for_service_start()
        for ip in self.master_server:
            try:
                log.info(Helper().CHECK_MIDDLEWARE_SERVICE.format(self.middleware_name, ip.ipaddress,
                                                                  self.minio_cfg["minio"]["port"]))
                requests.get(
                    "http://{IP}:{PORT}/minio/health/live".format(
                        IP=ip.ipaddress, PORT=self.minio_cfg["minio"]["port"]
                    )
                )
                log.info(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_SUCCEED.format(ip.ipaddress, self.minio_cfg["minio"]["port"]))
            except Exception as e:
                log.info(Helper().CHECK_MIDDLEWARE_SERVICE_PORT_FAILED.format(ip.ipaddress, self.minio_cfg["minio"]["port"]))
                log.error(e)
                exit(2)

    @status_me("middleware")
    def deploy_minio(self):
        if self.check_is_deploy(self.minio_cfg):
            self._proxy_on_nginx()
            self.send_docker_compose_file()
            self.generate_conf_file()
            self.send_docker_conf_file()
            self.start()
            self.check()
            self._create_bucket("mysql-backup")
        self.create_minio_service_kubernetes()

    def deploy(self):
        self.deploy_minio()
